//
//  AudioWaveManager.m
//  AudioWave
//
//  Created by kinsuft173 on 2017/6/5.
//  Copyright © 2017年 Oudmon-iOS. All rights reserved.
//

#import "AudioWaveManager.h"
#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>

@interface AudioWaveManager()

@property (nonatomic, strong) SCSiriWaveformView *waveformView;
@property (nonatomic, strong) AVAudioPlayer *player;
@property (nonatomic, strong) CADisplayLink* displayLink;
@end

@implementation AudioWaveManager

RCT_EXPORT_MODULE()

- (UIView *)view
{
    NSError *error;
//    self.player = [[AVAudioPlayer alloc] initWithContentsOfURL:[[NSBundle mainBundle] URLForResource:@"sample" withExtension:@"m4a"] error:&error];
    
        [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryPlayAndRecord error:&error];
    self.displayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(updateMeters)];
    [self.displayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
    
    self.waveformView = [[SCSiriWaveformView alloc] init];
    [self.waveformView setWaveColor:[UIColor whiteColor]];
    [self.waveformView setPrimaryWaveLineWidth:3.0f];
    [self.waveformView setSecondaryWaveLineWidth:1.5];
    self.waveformView.delegate = self;
    
    return self.waveformView;
}

- (void)updateMeters
{
    CGFloat normalizedValue;

    [self.player updateMeters];
    normalizedValue = [self _normalizedPowerLevelFromDecibels:[self.player averagePowerForChannel:0]];
    
    [self.waveformView updateWithLevel:normalizedValue];
}

#pragma mark - Private

- (CGFloat)_normalizedPowerLevelFromDecibels:(CGFloat)decibels
{
    if (decibels < -60.0f || decibels == 0.0f) {
        return 0.0f;
    }
    
    return powf((powf(10.0f, 0.05f * decibels) - powf(10.0f, 0.05f * -60.0f)) * (1.0f / (1.0f - powf(10.0f, 0.05f * -60.0f))), 1.0f / 2.0f);
}

//View Delegate
- (void)playSource:(NSString*)str
{
    if (self.player) {
        [self.player stop];
    }
    
    NSError *error;
    NSString* strUrl = [NSString stringWithFormat:@"%@Hz",str];
    self.player = [[AVAudioPlayer alloc] initWithContentsOfURL:[[NSBundle mainBundle] URLForResource:strUrl withExtension:@"m4a"] error:&error];
    self.player.numberOfLoops = -1;
    
    if (error) {
        return;
    }else {
        [self.player prepareToPlay];
        [self.player setMeteringEnabled:YES];
        [self.player play];
    }
    
}

- (void)close
{
    if (self.player) {
        [self.player stop];
    }
    
    self.displayLink.paused = YES;
    [self.displayLink invalidate];
    self.displayLink = nil;
}


RCT_EXPORT_VIEW_PROPERTY(playSource, NSString)
RCT_EXPORT_VIEW_PROPERTY(close, BOOL)

@end
